import { __awaiter, __decorate } from "tslib";
import { LifecycleMixin } from '@genesislcap/foundation-utils';
import { customElement, FoundationElement } from '@genesislcap/web-core';
import { Subscription } from 'rxjs';
import { FDC3 } from '../fdc3';
/**
 * fdc3-listener
 * @alpha
 */
let Fdc3Listener = class Fdc3Listener extends LifecycleMixin(FoundationElement) {
    constructor() {
        super(...arguments);
        this.intentConfig = [];
        this.channelConfig = [];
        this.channelSubscription = new Subscription();
    }
    deepClone() {
        const copy = super.deepClone();
        if (this.intentConfig) {
            copy.intentConfig = this.intentConfig.map((c) => {
                const copyConfig = Object.assign({}, c);
                if (c.callback) {
                    copyConfig.callback = c.callback.bind(this);
                }
                return copyConfig;
            });
        }
        if (this.channelConfig) {
            copy.channelConfig = this.channelConfig.map((c) => {
                const copyConfig = Object.assign({}, c);
                if (c.callback) {
                    copyConfig.callback = c.callback.bind(this);
                }
                return copyConfig;
            });
        }
        return copy;
    }
    connectedCallback() {
        super.connectedCallback();
        if (window.fdc3) {
            const listeners = new Map();
            this.intentConfig.forEach((intent) => listeners.set(intent.intent, intent.callback));
            this.fdc3.addIntentListeners(listeners);
            this.channelConfig.forEach((c) => __awaiter(this, void 0, void 0, function* () {
                if (c.channelName) {
                    const channel = yield this.fdc3.getOrCreateChannel(c.channelName);
                    this.handleChanelCurrentContext(channel, c.channelType, c.callback);
                    this.fdc3.addChannelListener(channel, c.channelType, c.callback);
                }
                else {
                    this.channelSubscription.add(this.fdc3.currentChannelObs.subscribe((channel) => {
                        this.handleChanelCurrentContext(channel, c.channelType, c.callback);
                        this.fdc3.addChannelListener(channel, c.channelType, c.callback);
                    }));
                }
            }));
        }
    }
    disconnectedCallback() {
        this.channelSubscription.unsubscribe();
        super.disconnectedCallback();
    }
    handleChanelCurrentContext(channel, contextType, listenerCallback) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentContextValue = yield channel.getCurrentContext(contextType);
            if (currentContextValue) {
                listenerCallback(currentContextValue);
            }
        });
    }
};
__decorate([
    FDC3
], Fdc3Listener.prototype, "fdc3", void 0);
Fdc3Listener = __decorate([
    customElement({
        name: 'fdc3-listener',
    })
], Fdc3Listener);
export { Fdc3Listener };
